/*	CodeEdit - A programmers editor for Atheos
	Copyright (c) 2002 Andreas Engh-Halstvedt

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.
	
	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
	MA 02111-1307, USA
*/
#ifndef _EDITWIN_H_
#define _EDITWIN_H_

#include <gui/window.h>

#include <string>

namespace os{
	class Menu;
	class FileRequester;
};
class App;
class CodeView;
class StatusBar;
class FindDialog;
class GotoLineDialog;

class EditWin : public os::Window{
public:
	enum{ ID_NOTHING,
	      ID_ABOUT, ID_QUIT, ID_OPTIONS,
	      
	      ID_FILE_NEW, ID_FILE_OPEN, ID_FILE_SAVE, ID_FILE_SAVE_AS,
	      ID_FILE_CLOSE, ID_DO_FILE_SAVE_AS, ID_DO_FILE_SAVE,
	      
	      ID_EDIT_UNDO, ID_EDIT_REDO, ID_EDIT_CUT, ID_EDIT_COPY,
	      ID_EDIT_PASTE, ID_EDIT_DELETE,
	      
	      ID_TEXT_INVOKED,
	      
	      ID_FIND, ID_FIND_NEXT, ID_FIND_PREVIOUS,
	      ID_REPLACE, ID_REPLACE_NEXT, ID_REPLACE_ALL,
	      ID_GOTO_LINE,
	      ID_DO_FIND, ID_DO_REPLACE, ID_DO_GOTO_LINE,
	      
	      ID_FORMAT_CHANGED
	};
	
private:	
	static uint unnamedCounter;
	
	string title;
	bool changed;
	bool named;
	
	App* app;
	os::Menu* menu;
	StatusBar* statusbar;
	
	CodeView* edit;
	
	string searchText;
	bool searchDown, searchCase, searchExtended;
	FindDialog* findDialog;
	
	GotoLineDialog* gotoLineDialog;
	
	os::FileRequester *saveRequester;
	bool saveRequesterVisible;
	
	void appAbout();
	void appQuit();
	void appOptions();
	
	void fileNew();
	void fileOpen();
	bool fileSave(const string &path="");
	void fileSaveAs(os::Message *parent=NULL);
	void fileClose();
	
	void editUndo();
	void editRedo();
	void editCopy();
	void editCut();
	void editPaste();
	void editDelete();
	
	void formatChanged();
	
	void find(const string&, bool, bool, bool);
	
	void updateTitle();
	void updateStatus(const string &s="", bigtime_t =5000000);
	bool load(const string &);
public:
	EditWin(App*, const char* =NULL);
	~EditWin();
	
	//overridden methods
	virtual bool OkToQuit();
	void DispatchMessage(os::Message*, os::Handler*);
	void HandleMessage(os::Message*);
};

#endif










